package info.textgrid.middleware;

import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListIdentifiersType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;
import info.textgrid.namespaces.middleware.tgcrud.common.TextGridMimetypes;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.Map;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;



/**
 * Class to build the Element for a ListIdentifiers request
 * 
 * @author Maximilian Brodhun, SUB
 * @version 1.0
 * @since 20.02.2014
 */

public class IdentifierListDelivererDC extends IdentifierListDelivererAbstract {

  private static org.apache.commons.logging.Log log =
      LogFactory.getLog(IdentifierListDelivererDC.class);
  private String datestamp;
  private boolean idExist = true;
  private OAI_ESClient oaiEsClient;

  private String[] identifierListFields; // Fields for the elasticsearch request
  private String rangeField; // Field for the optional range query
  private String dateOfObjectCreation; // Field for the object creation in the repository
  private String repositoryObjectURIPrefix; //
  private String identifierField;
  private static final int lifeTimeResToken = 10000;

  public static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  private int searchResponseSize;

  /**
   * In OAIPMH a ListIdentifiers request is answered by responding the datestamp and the identifier
   * of an object Both will be responded in a header element. This function build the header element
   * 
   * @param datestamp taken from the elasticSearch index (created element)
   * @param identifier taken from the elasticSearch index (textgridUri)
   * @return the ListIdentifierElement with the responded header element
   */

  public IdentifierListDelivererDC(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }



  /**
   * @param datestamp
   * @param identifier
   * @param lit
   * @param set
   * @return
   */
  private static ListIdentifiersType setListIdentifierHeader(String datestamp, String identifier,
      ListIdentifiersType lit, String set) {

    // FIXME Delete this function and check for empty set string in setListIdentifierHeaderWithSet

    HeaderType header = new HeaderType();
    header.setDatestamp(datestamp);
    header.setIdentifier(identifier);

    if (set != null) {
      header.getSetSpec().add(set);
    }
    lit.getHeader().add(header);

    return lit;
  }

  /**
   * To get the required values for the ListIdentifiers request this function will ask ElasticSearch
   * for a specific textgridUri the values "created" and "textgridUri".
   * 
   * Since the ListIdentifiers request enables the possibility the limit the request in a specific
   * time interval, it is necessary to perform a range query on the ElasticSearch index
   * 
   * @param from : start value for the range query
   * @param to : end value to the range query
   * @return after calling the function "setListIdentifierHeader" the return value is the whole
   *         ListIdentifiers element
   */
  public ListIdentifiersType processIdentifierList(String from, String to, String set,
      String resumptionToken) {

    // FIXME combine it with with getIdentifierListWithSet and check for empty set

    ListIdentifiersType lit = new ListIdentifiersType();
    QueryBuilder query = setOrNot(set, from, to);
    SearchResponse listListIdentiferValues;

    if (resumptionToken != null) {
      listListIdentiferValues = OAI_ESClient.getOaiESClient().prepareSearchScroll(resumptionToken)
          .setScroll(new TimeValue(lifeTimeResToken)).execute().actionGet();
    } else {
      listListIdentiferValues = OAI_ESClient
          .getOaiESClient().prepareSearch(OAI_ESClient.getEsIndex())
          .setTypes(OAI_ESClient.getEsType())
          .setScroll(new TimeValue(lifeTimeResToken))
          .addFields(this.identifierListFields)
          .setQuery(query)
          .setSize(this.searchResponseSize)
          .execute()
          .actionGet();
    }
    listListIdentiferValues =
        hitHandling(listListIdentiferValues, lit, set, listListIdentiferValues.getScrollId());

    return lit;
  }

  /**
   * @param listFurtherValues
   * @param lit
   * @param set
   * @param resumptionToken
   * @return
   */
  public SearchResponse hitHandling(SearchResponse listFurtherValues, ListIdentifiersType lit,
      String set, String resumptionToken) {

    int i = 0;
    for (SearchHit hit : listFurtherValues.getHits().getHits()) {
      i++;
      if (this.textgrid) {

        String datestamp = hit.getFields().get(this.dateOfObjectCreation).getValue().toString();
        try {
          datestamp = OAIPMHUtillities.convertDateFormat(datestamp).toXMLFormat();
        } catch (ParseException e1) {
          log.debug(e1);
        } catch (DatatypeConfigurationException e1) {
          log.debug(e1);
        }
        String identifier = hit.getFields().get(this.identifierField).getValue().toString();

        lit = setListIdentifierHeader(datestamp, identifier, lit, set);
      }

      if (this.dariah) {

        this.datestamp = hit.getFields().get(this.dateOfObjectCreation).getValue().toString();

        try {
          this.datestamp = hit.getFields().get(this.dateOfObjectCreation).getValue().toString();
          this.datestamp = OAIPMHUtillities.convertDateFormat(this.datestamp).toXMLFormat();
        } catch (ParseException e1) {
          log.debug(e1);
        } catch (DatatypeConfigurationException e1) {
          log.debug(e1);
        }

        String identifier = hit.getFields().get(this.identifierField).getValue().toString();
        lit = setListIdentifierHeader(this.datestamp, identifier, lit, set);
      }
    }

    // Check the need for a resumption token!
    ResumptionTokenType resTokenForResponse = OAIPMHUtillities.getResumptionToken(
        listFurtherValues.getHits().getTotalHits(), resumptionToken, cursorCollector,
        listFurtherValues.getScrollId(), this.searchResponseSize, i);
    if (resTokenForResponse != null) {
      lit.setResumptionToken(resTokenForResponse);
    }

    return listFurtherValues;
  }

  /**
   * @param set
   * @param from
   * @param to
   * @return
   */
  public QueryBuilder setOrNot(String set, String from, String to) {

    String filterFormat = "";
    if (this.textgrid == true) {
      filterFormat = TextGridMimetypes.EDITION;
    } else {
      filterFormat = TextGridMimetypes.DARIAH_COLLECTION;
    }

    QueryBuilder rangeQuery = QueryBuilders.rangeQuery(this.rangeField).from(from).to(to);
    QueryBuilder formatQuery = QueryBuilders.matchPhrasePrefixQuery("format", filterFormat);
    QueryBuilder query;

    if (set != null) {
      String queryField = "";
      String valueField = "";

      if (this.dariah == true) {
        queryField = "descriptiveMetadata.dc:relation";
        valueField = set;
      }

      if (this.textgrid == true) {
        String[] setParts = set.split(":");
        if (setParts[0].equals("project")) {
          queryField = "project.id";
          valueField = setParts[1];
        }
      }

      QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(queryField, valueField);
      if (this.dariah) {
        query = matchQuery;
      } else {
        query = QueryBuilders.boolQuery().must(matchQuery).must(formatQuery);
      }

    } else {
      if (this.dariah == true)
        query = rangeQuery;
      else
        query = QueryBuilders.boolQuery().must(rangeQuery).must(formatQuery);
    }

    return query;
  }

  /**
   * @param lit
   * @return
   */
  public ListIdentifiersType getLit(ListIdentifiersType lit) {
    return lit;
  }

  /**
   * @return
   */
  public String getDateOfObjectCreation() {
    return this.dateOfObjectCreation;
  }

  /**
   * @param dateOfObjectCreation
   */
  public void setDateOfObjectCreation(String dateOfObjectCreation) {
    this.dateOfObjectCreation = dateOfObjectCreation;
  }

  /**
   * @param repositoryObjectURIPrefix
   */
  public void setRepositoryObjectURIPrefix(String repositoryObjectURIPrefix) {
    this.repositoryObjectURIPrefix = repositoryObjectURIPrefix;
  }

  /**
   * @return
   */
  public String getRepositoryObjectURIPrefix() {
    return this.repositoryObjectURIPrefix;
  }

  /**
   * @param validItem
   */
  public void setIdExist(boolean validItem) {
    this.idExist = validItem;
  }

  /**
   * @return
   */
  public boolean getIdExist() {
    return this.idExist;
  }

  /**
   * @return
   */
  public String[] getIdentifierListFields() {
    return this.identifierListFields;
  }

  /**
   * @param identifierListFields
   */
  public void setIdentifierListFields(String[] identifierListFields) {
    this.identifierListFields = identifierListFields;
  }

  /**
   * @return
   */
  public String getFieldForRange() {
    return this.rangeField;
  }

  /**
   * @param fieldForRange
   */
  public void setFieldForRange(String fieldForRange) {
    this.rangeField = fieldForRange;
  }

  /**
   * @return
   */
  public String getIdentifierField() {
    return this.identifierField;
  }

  /**
   * @param identifierPrefix
   */
  public void setIdentifierField(String identifierPrefix) {
    this.identifierField = identifierPrefix;
  }

  /**
   * @return
   */
  public int getSearchResponseSize() {
    return this.searchResponseSize;
  }

  /**
   * @param searchResponseSize
   */
  public void setSearchResponseSize(String searchResponseSize) {
    this.searchResponseSize = Integer.parseInt(searchResponseSize);
  }

  /**
   * @return
   */
  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  /**
   * @param oaiEsClient
   */
  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

}
