package info.textgrid.middleware;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.Map;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetRequestBuilder;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.Client;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 *
 */
public class RecordListDelivererDC extends RecordListDelivererAbstract {

  // **
  // STATICS
  // **

  private static org.apache.commons.logging.Log log = LogFactory.getLog(OAIPMHImpl.class);

  // **
  // CLASS VARIABLES
  // **

  private OAI_ESClient oaiEsClient;
  private String[] fields;
  private String[] workFields;
  private String formatField;
  private String formatToFilter;
  private String dateOfObjectCreation;
  private String relationToFurtherMetadataObject;
  private String repositoryObjectURIPrefix;
  private String rangeField;
  private String modifiedField;
  private String identifierField;
  private boolean foundItems;

  // **
  // DC-Field Lists
  // **

  private String[] contributorList;
  private String[] coverageList;
  private String[] creatorList;
  private String[] dateList;
  private String[] descriptionList;
  private String[] formatList;
  private String[] identifierList;
  private String[] languageList;
  private String[] publisherList;
  private String[] relationList;
  private String[] relationForWorkList;
  private String[] rightList;
  private String[] sourceList;
  private String[] subjectList;
  private String[] titleList;
  private String[] typeList;
  private String modifiedValue;
  private static final int lifeTimeResToken = 60000;

  static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  private int searchResponseSize;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererDC(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   * <p>
   * The furtherDCElementes fetches the specified fields from the work object. This function is
   * called from the function fetchFields.
   * </p>
   * 
   * @param id
   * @param client
   * @return
   */
  public GetResponse furtherDCElements(String id, Client client) {

    if (id != null) {
      try {
        id = URLDecoder.decode(id, "UTF-8");
      } catch (UnsupportedEncodingException e) {
        e.printStackTrace();
      }

      GetRequestBuilder getWorkValues =
          OAI_ESClient.getOaiESClient()
              .prepareGet()
              .setIndex(OAI_ESClient.getEsIndex())
              .setType(OAI_ESClient.getEsType())
              .setId(id)
              .setFields(this.workFields);

      GetResponse responseWorkValues = getWorkValues.execute().actionGet();

      return responseWorkValues;
    }

    return null;
  }

  /**
   * <p>
   * The fetchFields function filters all edition and fetch the specified fields
   * </p>
   * 
   * @param query
   * @param recordList
   * @param resumptionToken
   * @param set
   * @throws UnsupportedEncodingException
   */
  public void fetchFields(QueryBuilder query, ListRecordsType recordList, String resumptionToken,
      String set) throws UnsupportedEncodingException {

    String scrollID = "";
    SearchResponse scrollResp;
    QueryBuilder recordFilter;
    DublinCoreBuilder dublinCoreBuilder = new DublinCoreBuilder();
    if (this.dariah == true) {
      recordFilter = query;
    } else {
      recordFilter = QueryBuilders.boolQuery().must(query)
          .must(QueryBuilders.matchPhraseQuery(this.formatField, this.formatToFilter));
    }

    if (resumptionToken == null) {
      scrollResp = OAI_ESClient.getOaiESClient()
          .prepareSearch(OAI_ESClient.getEsIndex())
          .setScroll(new TimeValue(lifeTimeResToken))
          .setTypes(OAI_ESClient.getEsType())
          .addFields(this.fields)
          .setQuery(recordFilter)
          .setSize(this.searchResponseSize)
          .execute()
          .actionGet();
    } else {
      scrollResp = OAI_ESClient.getOaiESClient().prepareSearchScroll(resumptionToken)
          .setScroll(new TimeValue(lifeTimeResToken)).execute().actionGet();
    }

    log.info("Queried fields: " + this.fields.toString());
    log.info("Lifetime for resumption token is set to: " + lifeTimeResToken);

    scrollID = scrollResp.getScrollId();

    long completeListSize = scrollResp.getHits().totalHits();
    if (completeListSize > 0) {
      setFoundItems(true);

      int i = 0;

      for (SearchHit hit : scrollResp.getHits().getHits()) {
        i++;

        if (hit != null && hit.getFields() != null) {
          try {
            if (hit.getFields().get(this.dateOfObjectCreation) != null) {
              String datestamp =
                  hit.getFields().get(this.dateOfObjectCreation).getValue().toString();
              datestamp = OAIPMHUtillities.convertDateFormat(datestamp).toXMLFormat();
            }
          } catch (ParseException e1) {
            log.debug(e1);
          } catch (DatatypeConfigurationException e1) {
            log.debug(e1);
          }

          this.modifiedValue = hit.getFields().get(this.modifiedField).getValue().toString();

          // TextGrid search!
          String workUri = "";

          if (this.textgrid == true && hit.getFields().get(this.formatField).values().get(0)
              .toString().equals(this.formatToFilter)) {
            if (hit.getFields().get(this.relationToFurtherMetadataObject) != null) {
              workUri = hit.getFields().get(this.relationToFurtherMetadataObject).values().get(0)
                  .toString().substring(this.repositoryObjectURIPrefix.length());

              if (!workUri.endsWith(".0")) {
                workUri = workUri.concat(".0");
              }
            }

            dublinCoreBuilder = putContentIntoDCFieldLists(hit,
                furtherDCElements(workUri, OAI_ESClient.getOaiESClient()));

            buildRecord(recordList, set,
                hit.getFields().get(this.identifierField).getValue().toString(), dublinCoreBuilder);
          }

          if (this.dariah == true) {
            dublinCoreBuilder = putContentIntoDCFieldLists2(hit,
                furtherDCElements(hit.getId(), OAI_ESClient.getOaiESClient()));
            buildRecord(recordList, set,
                hit.getFields().get(this.identifierField).getValue().toString(), dublinCoreBuilder);
          }
        }
      }

      // Check the need for a resumption token!
      ResumptionTokenType resTokenForResponse = OAIPMHUtillities.getResumptionToken(
          completeListSize, resumptionToken, cursorCollector, scrollID, this.searchResponseSize, i);
      if (resTokenForResponse != null) {
        recordList.setResumptionToken(resTokenForResponse);
      }

    } else {
      setFoundItems(false);
    }
  }

  /**
   * @param hit
   * @param responseWorkValues
   * @return
   */
  public DublinCoreBuilder putContentIntoDCFieldLists(SearchHit hit,
      GetResponse responseWorkValues) {

    DublinCoreBuilder result = new DublinCoreBuilder();

    result.setContributor(
        DublinCoreFieldLoader.setContributor(responseWorkValues, this.contributorList));
    result.setCoverage(DublinCoreFieldLoader.setCoverage(responseWorkValues, this.coverageList));
    result.setCreator(DublinCoreFieldLoader.setCreator(responseWorkValues, this.creatorList));
    result.setDate(DublinCoreFieldLoader.setDate(responseWorkValues, this.dateList));
    result.setDescription(
        DublinCoreFieldLoader.setDescription(responseWorkValues, this.descriptionList));
    result.setFormat(DublinCoreFieldLoader.setFormat(hit, this.formatList));
    result.setIdentifier(DublinCoreFieldLoader.setIdentifier(hit, this.identifierList));
    result.setLanguage(DublinCoreFieldLoader.setLanguage(hit, this.languageList));
    result.setPublisher(DublinCoreFieldLoader.setPublisher(hit, this.publisherList));
    result.setRelation(DublinCoreFieldLoader.setRelation(hit, this.relationList));
    result.setRelation(
        DublinCoreFieldLoader.setRelationForWork(responseWorkValues, this.relationForWorkList));
    result.setRights(DublinCoreFieldLoader.setRights(hit, this.rightList));
    result.setSource(DublinCoreFieldLoader.setSources(hit, this.sourceList));
    result.setSubject(DublinCoreFieldLoader.setSubject(hit, this.subjectList));
    result.setTitle(DublinCoreFieldLoader.setTitle(hit, this.titleList));
    result.setType(DublinCoreFieldLoader.setType(responseWorkValues, this.typeList));

    return result;
  }

  /**
   * @param hit
   * @param responseWorkValues
   * @return
   */
  public DublinCoreBuilder putContentIntoDCFieldLists2(SearchHit hit,
      GetResponse responseWorkValues) {

    DublinCoreBuilder result = new DublinCoreBuilder();

    result.setContributor(DublinCoreFieldLoader.setContributor(hit, this.contributorList));
    result.setCoverage(DublinCoreFieldLoader.setCoverage(hit, this.coverageList));
    result.setCreator(DublinCoreFieldLoader.setCreator(hit, this.creatorList));
    result.setDate(DublinCoreFieldLoader.setDate(hit, this.dateList));
    result.setDescription(DublinCoreFieldLoader.setDescription(hit, this.descriptionList));
    result.setFormat(DublinCoreFieldLoader.setFormat(hit, this.formatList));
    result.setIdentifier(DublinCoreFieldLoader.setIdentifier(hit, this.identifierList));
    result.setLanguage(DublinCoreFieldLoader.setLanguage(hit, this.languageList));
    result.setPublisher(DublinCoreFieldLoader.setPublisher(hit, this.publisherList));
    result.setRelation(DublinCoreFieldLoader.setRelation(hit, this.relationList));
    result.setRights(DublinCoreFieldLoader.setRights(hit, this.rightList));
    result.setSource(DublinCoreFieldLoader.setSources(hit, this.sourceList));
    result.setSubject(DublinCoreFieldLoader.setSources(hit, this.subjectList));
    result.setTitle(DublinCoreFieldLoader.setTitle(hit, this.titleList));
    result.setType(DublinCoreFieldLoader.setType(hit, this.typeList));

    return result;
  }

  /**
   * @param recordList
   * @param set
   * @param headerIdentifier
   * @param dublinCoreBuilder
   * @return
   */
  public RecordType buildRecord(ListRecordsType recordList, String set, String headerIdentifier,
      DublinCoreBuilder dublinCoreBuilder) {

    MetadataType metadata = new MetadataType();
    RecordType record = new RecordType();

    metadata = dublinCoreBuilder.getDC();
    record.setMetadata(metadata);
    record.setHeader(setHeader(set, headerIdentifier));
    recordList.getRecord().add(record);

    return record;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#getRecords(java.lang.String,
   * java.lang.String, java.lang.String, java.lang.String)
   */
  public ListRecordsType getRecords(String from, String to, String set, String resumptionToken) {

    log.info("Started List Records with booleans for DARIAH:  " + this.dariah + " and TextGrid: "
        + this.textgrid);

    ListRecordsType recordList = new ListRecordsType();
    QueryBuilder query;
    QueryBuilder rangeQuery = QueryBuilders.rangeQuery(this.rangeField).from(from).to(to);

    if (set != null) {

      String queryField = "";
      String valueField = "";

      if (this.dariah == true) {
        queryField = "descriptiveMetadata.dc:relation";
        valueField = set;
      }

      if (this.textgrid == true) {
        String[] setParts = set.split(":");
        if (setParts[0].equals("project")) {
          queryField = "project.id";
          valueField = setParts[1];
        }
      }

      QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(queryField, valueField);
      QueryBuilder boolQuery = QueryBuilders.boolQuery().must(rangeQuery).must(matchQuery);

      query = boolQuery;
    } else {
      query = rangeQuery;
    }
    try {
      fetchFields(query, recordList, resumptionToken, set);
    } catch (UnsupportedEncodingException e) {
      e.printStackTrace();
    }

    return recordList;
  }

  /**
   * @param set
   * @param headerIdentifier
   * @return
   */
  public HeaderType setHeader(final String set, final String headerIdentifier) {

    HeaderType header = new HeaderType();
    String identifierForHeader = headerIdentifier;

    // Set date in XML format.
    try {
      header.setDatestamp(OAIPMHUtillities.convertDateFormat(this.modifiedValue).toXMLFormat());
    } catch (ParseException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } catch (DatatypeConfigurationException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    // Set header specific for TextGrid objects (remove URL).
    if (this.textgrid == true) {
      identifierForHeader = identifierForHeader.replaceFirst("https://textgridrep.org/", "");
    }

    header.setIdentifier(identifierForHeader);

    // Set set :-)
    if (set != null) {
      header.getSetSpec().add(set);
    }

    return header;
  }

  /**
   * @param formatToFilter
   */
  public void setFormatToFilter(String formatToFilter) {
    this.formatToFilter = formatToFilter;
  }

  /**
   * @param formatField
   */
  public void setFormatField(String formatField) {
    this.formatField = formatField;
  }

  /**
   * @return
   */
  public String getDateOfObjectCreation() {
    return this.dateOfObjectCreation;
  }

  /**
   * @param dateOfObjectCreation
   */
  public void setDateOfObjectCreation(String dateOfObjectCreation) {
    this.dateOfObjectCreation = dateOfObjectCreation;
  }

  /**
   * @return
   */
  public String getRelationToFurtherMetadataObject() {
    return this.relationToFurtherMetadataObject;
  }

  /**
   * @param relationToFurtherMetadataObject
   */
  public void setRelationToFurtherMetadataObject(String relationToFurtherMetadataObject) {
    this.relationToFurtherMetadataObject = relationToFurtherMetadataObject;
  }

  /**
   * @return
   */
  public String getRepositoryObjectURIPrefix() {
    return this.repositoryObjectURIPrefix;
  }

  /**
   * @param repositoryObjectURIPrefix
   */
  public void setRepositoryObjectURIPrefix(String repositoryObjectURIPrefix) {
    this.repositoryObjectURIPrefix = repositoryObjectURIPrefix;
  }

  /**
   * @return
   */
  public String getFieldForRange() {
    return this.rangeField;
  }

  /**
   * @param fieldForRange
   */
  public void setFieldForRange(String fieldForRange) {
    this.rangeField = fieldForRange;
  }

  /*
   * Setter for DublinCore Lists
   */

  /**
   * @param contributorList
   */
  public void setContributor(String[] contributorList) {
    this.contributorList = contributorList;
  }

  /**
   * @param coveragesList
   */
  public void setCoverage(String[] coveragesList) {
    this.coverageList = coveragesList;
  }

  /**
   * @param creatorsList
   */
  public void setCreators(String[] creatorsList) {
    this.creatorList = creatorsList;
  }

  /**
   * @param datesList
   */
  public void setDates(String[] datesList) {
    this.dateList = datesList;
  }

  /**
   * @param descriptionsList
   */
  public void setDescriptions(String[] descriptionsList) {
    this.descriptionList = descriptionsList;
  }

  /**
   * @param formatsList
   */
  public void setFormats(String[] formatsList) {
    this.formatList = formatsList;
  }

  /**
   * @param identifiersList
   */
  public void setIdentifiers(String[] identifiersList) {
    this.identifierList = identifiersList;
  }

  /**
   * @param languagesList
   */
  public void setLanguages(String[] languagesList) {
    this.languageList = languagesList;
  }

  /**
   * @param publishersList
   */
  public void setPublishers(String[] publishersList) {
    this.publisherList = publishersList;
  }

  /**
   * @param relationsList
   */
  public void setRelations(String[] relationsList) {
    this.relationList = relationsList;
  }

  /**
   * @param relationsList
   */
  public void setRelationsForWork(String[] relationsList) {
    this.relationForWorkList = relationsList;
  }

  /**
   * @param rightsList
   */
  public void setRights(String[] rightsList) {
    this.rightList = rightsList;
  }

  /**
   * @param sourcesList
   */
  public void setSources(String[] sourcesList) {
    this.sourceList = sourcesList;
  }

  /**
   * @param subjectsList
   */
  public void setSubjects(String[] subjectsList) {
    this.subjectList = subjectsList;
  }

  /**
   * @param titlesList
   */
  public void setTitles(String[] titlesList) {
    this.titleList = titlesList;
  }

  /**
   * @param typesList
   */
  public void setTypes(String[] typesList) {
    this.typeList = typesList;
  }

  /**
   * @return
   */
  public boolean isFoundItems() {
    return this.foundItems;
  }

  /**
   * @param foundItems
   */
  public void setFoundItems(boolean foundItems) {
    this.foundItems = foundItems;
  }

  /**
   * @return
   */
  public String getModifiedField() {
    return this.modifiedField;
  }

  /**
   * @param modifiedField
   */
  public void setModifiedField(String modifiedField) {
    this.modifiedField = modifiedField;
  }

  /**
   * @return
   */
  public String getIdentifierField() {
    return this.identifierField;
  }

  /**
   * @param identifierField
   */
  public void setIdentifierField(String identifierField) {
    this.identifierField = identifierField;
  }

  /**
   * @return
   */
  public int getSearchResponseSize() {
    return this.searchResponseSize;
  }

  /**
   * @param searchResponseSize
   */
  public void setSearchResponseSize(String searchResponseSize) {
    this.searchResponseSize = Integer.parseInt(searchResponseSize);
  }

  /**
   * @return
   */
  public boolean getTextGrid() {
    return this.textgrid;
  }

  /**
   * @return
   */
  public String[] getWorkFields() {
    return this.workFields;
  }

  /**
   * @param workFields
   */
  public void setWorkFields(String[] workFields) {
    this.workFields = workFields;
  }

  /**
   * @param fields
   */
  public void setFields(String[] fields) {
    this.fields = fields;
  }

  /**
   * @return
   */
  public String[] getFields() {
    return this.fields;
  }

  /**
   * @return
   */
  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  /**
   * @param oaiEsClient
   */
  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

}
