package info.textgrid.middleware;

import java.text.ParseException;
import javax.xml.bind.JAXB;
import javax.xml.bind.JAXBElement;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.oaipmh.DeletedRecordType;
import info.textgrid.middleware.oaipmh.GranularityType;
import info.textgrid.middleware.oaipmh.OAIPMHtype;
import info.textgrid.namespaces.middleware.tgcrud.common.TextGridMimetypes;

/**
 *
 */
@Ignore
public class OaiPmhDARIAHTest {

  public static OAI_ESClient oaiEsClient;
  DeletedRecordType deletedRecordType = DeletedRecordType.NO;
  GranularityType granularityType = GranularityType.YYYY_MM_DD_THH_MM_SS_Z;

  private static RecordDelivererDC record;
  private static RecordListDelivererDC recordList;
  private static IdentifierListDelivererDC identifierList;
  private static MetadataFormatListDeliverer metadataFormatList = new MetadataFormatListDeliverer();
  private static SetDeliverer setListDARIAH = new SetDeliverer(false, true);

  private RepIdentification rep = new RepIdentification("DARIAH-DE Repository",
      "http://trep.de.dariah.eu", this.deletedRecordType, "2011-06-11T02:32:40Z",
      this.granularityType, "2.0", "funk@sub.uni-goettingen.de");

  /**
   * FIXME: it should not be necessary to put idiom record variables for a DARIAH test case
   */
  private static RecordDelivererIDIOM recordIDIOM;
  private static RecordListDelivererIDIOM recordListIDIOM;
  private static IdentifierListDelivererIDIOM identifierListIDIOM =
      new IdentifierListDelivererIDIOM(true, false);

  private OAIPMHImpl request = new OAIPMHImpl(this.rep, OaiPmhDARIAHTest.record,
      OaiPmhDARIAHTest.recordIDIOM, OaiPmhDARIAHTest.recordList, OaiPmhDARIAHTest.recordListIDIOM,
      OaiPmhDARIAHTest.metadataFormatList, OaiPmhDARIAHTest.setListDARIAH,
      OaiPmhDARIAHTest.identifierList, OaiPmhDARIAHTest.identifierListIDIOM);

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUp() throws Exception {

    oaiEsClient = new OAI_ESClient("localhost", "9302", "dhrep-dariah-dev-public");
    OAI_ESClient.setEsIndex("dariah-public");
    OAI_ESClient.setEsType("metadata");

    record = new RecordDelivererDC(false, true);
    record.setOaiEsClient(oaiEsClient);
    record.setWorkFields(DARIAHConstants.DARIAHFields);
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetRequestIdentify() throws ParseException {

    System.out.println("Test for the verb \"Identify\" with succesfull response");
    JAXBElement<OAIPMHtype> r = this.request.getRequest("Identify", "", "", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetRequestGetRecordDariah() throws ParseException {

    OaiPmhDARIAHTest.record.setContributor(DARIAHConstants.CONTRIBUTOR_LIST);
    OaiPmhDARIAHTest.record.setCoverage(DARIAHConstants.COVERAGE_LIST);
    OaiPmhDARIAHTest.record.setCreator(DARIAHConstants.CREATOR_LIST);
    OaiPmhDARIAHTest.record.setDates(DARIAHConstants.DATE_LIST);
    OaiPmhDARIAHTest.record.setDescriptions(DARIAHConstants.DESCRIPTION_LIST);
    OaiPmhDARIAHTest.record.setFormats(DARIAHConstants.FORMAT_LIST);
    OaiPmhDARIAHTest.record.setIdentifiers(DARIAHConstants.IDENTIFIER_LIST);
    OaiPmhDARIAHTest.record.setLanguages(DARIAHConstants.LANGUAGE_LIST);
    OaiPmhDARIAHTest.record.setPublishers(DARIAHConstants.PUBLISHER_LIST);
    OaiPmhDARIAHTest.record.setRelations(DARIAHConstants.RELATIONS_LIST);
    OaiPmhDARIAHTest.record.setRights(DARIAHConstants.RIGHTS_LIST);
    OaiPmhDARIAHTest.record.setSources(DARIAHConstants.SOURCE_LIST);
    OaiPmhDARIAHTest.record.setSubjects(DARIAHConstants.SUBJECT_LIST);
    OaiPmhDARIAHTest.record.setTitles(DARIAHConstants.TITLE_LIST);
    OaiPmhDARIAHTest.record.setTypes(DARIAHConstants.TYPE_LIST);
    OaiPmhDARIAHTest.record.setFields(DARIAHConstants.DARIAHFields);
    OaiPmhDARIAHTest.record.setFormatField(DARIAHConstants.FORMAT);
    OaiPmhDARIAHTest.record.setFormatToFilter("metadata");
    OaiPmhDARIAHTest.record.setDateOfObjectCreation(DARIAHConstants.MODIFIED_FIELD);
    OaiPmhDARIAHTest.record.setRepositoryObjectURIPrefix(DARIAHConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhDARIAHTest.record.setIdentifierField("administrativeMetadata.dcterms:identifier");

    System.out.println("Test for the verb \"GetRecord\" for DARIAH with succesfull response");
    JAXBElement<OAIPMHtype> p = this.request.getRequest("GetRecord",
        "hdl:21.T11991/0000-0005-E1AA-D", "oai_dc", "", "", "", "");
    JAXB.marshal(p, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testListRecordsDariah() throws ParseException {

    OaiPmhDARIAHTest.recordList.setContributor(DARIAHConstants.CONTRIBUTOR_LIST);
    OaiPmhDARIAHTest.recordList.setCoverage(DARIAHConstants.COVERAGE_LIST);
    OaiPmhDARIAHTest.recordList.setCreators(DARIAHConstants.CREATOR_LIST);
    OaiPmhDARIAHTest.recordList.setDates(DARIAHConstants.DATE_LIST);
    OaiPmhDARIAHTest.recordList.setDescriptions(DARIAHConstants.DESCRIPTION_LIST);
    OaiPmhDARIAHTest.recordList.setFormats(DARIAHConstants.FORMAT_LIST);
    OaiPmhDARIAHTest.recordList.setIdentifiers(DARIAHConstants.IDENTIFIER_LIST);
    OaiPmhDARIAHTest.recordList.setLanguages(DARIAHConstants.LANGUAGE_LIST);
    OaiPmhDARIAHTest.recordList.setPublishers(DARIAHConstants.PUBLISHER_LIST);
    OaiPmhDARIAHTest.recordList.setRelations(DARIAHConstants.RELATIONS_LIST);
    OaiPmhDARIAHTest.recordList.setRights(DARIAHConstants.RIGHTS_LIST);
    OaiPmhDARIAHTest.recordList.setSources(DARIAHConstants.SOURCE_LIST);
    OaiPmhDARIAHTest.recordList.setSubjects(DARIAHConstants.SUBJECT_LIST);
    OaiPmhDARIAHTest.recordList.setTitles(DARIAHConstants.TITLE_LIST);
    OaiPmhDARIAHTest.recordList.setTypes(DARIAHConstants.TYPE_LIST);
    OaiPmhDARIAHTest.recordList.setFormatField(DARIAHConstants.FORMAT);
    OaiPmhDARIAHTest.recordList.setDateOfObjectCreation(DARIAHConstants.CREATED);
    OaiPmhDARIAHTest.recordList.setFieldForRange(DARIAHConstants.MODIFIED_FIELD);
    OaiPmhDARIAHTest.recordList.setDateOfObjectCreation(DARIAHConstants.RANGE_FIELD);
    OaiPmhDARIAHTest.recordList.setIdentifierField(DARIAHConstants.IDENTIFIER);
    OaiPmhDARIAHTest.recordList.setFormatToFilter(TextGridMimetypes.DARIAH_COLLECTION);
    OaiPmhDARIAHTest.recordList.setModifiedField(DARIAHConstants.MODIFIED_FIELD);
    OaiPmhDARIAHTest.recordList
        .setRelationToFurtherMetadataObject("descriptiveMetadata.dc:relation");
    OaiPmhDARIAHTest.recordList.setSearchResponseSize("100");

    JAXBElement<OAIPMHtype> r = this.request.getRequest("ListRecords", "", "oai_dc",
        "21.T11991/0000-0003-718E-E", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testListSetsDARIAH() throws ParseException {

    OaiPmhDARIAHTest.setListDARIAH.setFormatField(DARIAHConstants.FORMAT);
    OaiPmhDARIAHTest.setListDARIAH.setFormatToFilter(DARIAHConstants.COLLECTION_MIMETYPE);
    OaiPmhDARIAHTest.setListDARIAH.setIdentifierField(DARIAHConstants.IDENTIFIER);
    OaiPmhDARIAHTest.setListDARIAH
        .setRepositoryObjectURIPrefix(DARIAHConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhDARIAHTest.setListDARIAH.setIdentifierField("administrativeMetadata.dcterms:identifier");
    OaiPmhDARIAHTest.setListDARIAH.setSpecField("descriptiveMetadata.dc:title");
    OaiPmhDARIAHTest.setListDARIAH.setSpecFieldPrefix(DARIAHConstants.COLLECTIONREGISTRY_PREFIX);
    JAXBElement<OAIPMHtype> r = this.request.getRequest("ListSets", "", "", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testListIdentifiersDARIAH() throws ParseException {

    OaiPmhDARIAHTest.identifierList.setFieldForRange(DARIAHConstants.RANGE_FIELD);
    OaiPmhDARIAHTest.identifierList.setIdentifierListFields(DARIAHConstants.IDENTIFIER_LIST_FIELDS);
    OaiPmhDARIAHTest.identifierList
        .setDateOfObjectCreation("administrativeMetadata.dcterms:modified");
    OaiPmhDARIAHTest.identifierList
        .setRepositoryObjectURIPrefix(DARIAHConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhDARIAHTest.identifierList.setIdentifierField("administrativeMetadata.dcterms:identifier");
    OaiPmhDARIAHTest.identifierList.setSearchResponseSize("100");
    JAXBElement<OAIPMHtype> r =
        this.request.getRequest("ListIdentifiers", "", "oai_dc", "", "", "", "");
    JAXB.marshal(r, System.out);
    System.out.println("-----------------------------------\n");
  }

}
